/*
  TwiKryepad.cpp

  This file is part of TwiKeypad library for Arduino.

  TwiKeypad library was developed by Synapse(Hiroshi Tanigawa) in 2017.
  This library is originally distributed at "Synapse's history of making
  gadgets." <https://synapse.kyoto>

  TwiKeypad library is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.

  TwiKeypad is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TwiKeypad library.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "TwiKeypad.h"
#include <arduino.h>
#include "Wire.h"

// Character assignment
PROGMEM const char TWIKEYPAD_4X4[16]={'*','0','#','D','7','8','9','C','4','5','6','B','1','2','3','A'};

char TwiKeypad::KeyToChar(int8_t key)
{
  if(key<0 || KeyChar==NULL) return '\0';
  if(rom) {
    return pgm_read_byte_near(KeyChar+key);
  } else {
    return KeyChar[key];
  }
} // TwiKeypad::KeyToChar

int16_t TwiKeypad::ReadReg(uint8_t RegAdr)
{
  if(ptw==NULL) return -1;
  ptw->beginTransmission(TwiAdr);
  ptw->write(RegAdr);
  ptw->endTransmission(false);
  ptw->requestFrom(TwiAdr,uint8_t(1));
  int16_t result=-1;
  while(ptw->available()) {
    result=ptw->read();
  } // while
  return result;
} // TwiKeypad::ReadReg

int8_t TwiKeypad::begin(TwoWire &tw, uint8_t adr, const char *_KeyChar, boolean _rom)
{
  ptw=&tw;
  TwiAdr=adr;
  KeyNum=GetKeyNum();
  if(_KeyChar==NULL && KeyNum==16) {
    KeyChar=TWIKEYPAD_4X4;
    rom=true;
  } else {
    KeyChar=_KeyChar;
    rom=_rom;
  }
  int8_t result=ClearBuffer();
  if(result==0) result=SetRepeat(0,0);
  return result;
} // TwiKeypad::begin

int8_t TwiKeypad::GetKey()
{
  if(ptw==NULL) return -1;
  int16_t result=ReadReg(0x07);
  if(result==0xff) result=-1;
  return int8_t(result);
} // TwiKeypad::GetKey

int8_t TwiKeypad::WaitForKey()
{
  if(ptw==NULL) return -1;
  int8_t result;
  do{
    result=GetKey();
  } while(result<0);
  return result;
} // TwiKeypad::WaitForKey

char TwiKeypad::GetChar()
{
  return KeyToChar(GetKey());
} // TwiKeypad::GetChar

char TwiKeypad::WaitForChar()
{
  return KeyToChar(WaitForKey());
} // TwiKeypad::WaitForChar

int8_t TwiKeypad::GetKeyNum()
{
  if(ptw==NULL) return -1;
  return ReadReg(0x02);
} // TwiKeypad::GetKeyNum

int16_t TwiKeypad::GetBoardId()
{
  if(ptw==NULL) return -1;
  return ReadReg(0x00);
} // TwiKeypad::GetBoardId

int16_t TwiKeypad::GetFirmVer()
{
  if(ptw==NULL) return -1;
  return ReadReg(0x01);
} // TwiKeypad::GetFirmVer

int8_t TwiKeypad::SetRepeat(uint8_t RepeatStart, uint8_t RepeatInterval)
{
  if(ptw==NULL) return -1;
  ptw->beginTransmission(TwiAdr);
  ptw->write(0x04);
  ptw->write(RepeatStart);
  ptw->write(RepeatInterval);
  ptw->endTransmission();
  return 0;
} // SetRepeat

int16_t TwiKeypad::GetRepeatStart()
{
  if(ptw==NULL) return -1;
  return ReadReg(0x04);
} // TwiKeypad::GetRepeatStart

int16_t TwiKeypad::GetRepeatInterval()
{
  if(ptw==NULL) return -1;
  return ReadReg(0x05);
} // TwiKeypad::GetRepeatInterval

int8_t TwiKeypad::GetBufferedKeyNum()
{
  if(ptw==NULL) return -1;
  return ReadReg(0x06);
} // TwiKeypad::GetBufferedKeyNum

int8_t TwiKeypad::ClearBuffer()
{
  if(ptw==NULL) return -1;
  ptw->beginTransmission(TwiAdr);
  ptw->write(0x06);
  ptw->write(0);
  ptw->endTransmission();
  return 0;
} // 

uint32_t TwiKeypad::GetKeyState()
{
  uint8_t bytes=(KeyNum+7)>>3;
  
  if(ptw==NULL) return -1;
  ptw->beginTransmission(TwiAdr);
  ptw->write(0x08);
  ptw->endTransmission(false);
  ptw->requestFrom(TwiAdr,uint8_t(bytes));
  uint32_t result=0;
  int8_t shifts=0;
  while(ptw->available()) {
    result+=uint32_t(ptw->read())<<shifts;
    shifts+=8;
  } // while
  return result;
} // TwiKeypad::GetKeyState
